package com.example.notifyservice;

import android.app.ActivityManager;
import android.app.Notification;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.IBinder;
import android.service.notification.NotificationListenerService;
import android.service.notification.StatusBarNotification;
import android.util.Log;
import java.util.Arrays;
import java.util.List;

public class Listener extends NotificationListenerService {


    List<String> notificationTextKeys = Arrays.asList(
            Notification.EXTRA_TEXT,
            Notification.EXTRA_SUB_TEXT,
            Notification.EXTRA_INFO_TEXT,
            Notification.EXTRA_SUMMARY_TEXT,
            Notification.EXTRA_BIG_TEXT,
            Notification.EXTRA_TEXT_LINES
    );


    @Override
    public IBinder onBind(Intent intent) {
        return super.onBind(intent);
    }

    private static String extractFirstNumber(String input) {
        if (input == null || input.isEmpty()) {
            return null;
        }

        StringBuilder number = new StringBuilder();
        boolean foundDigit = false;

        for (char c : input.toCharArray()) {
            if (Character.isDigit(c)) {
                number.append(c);
                foundDigit = true;
            } else if (foundDigit) {
                break;
            }
        }

        return number.length() > 0 ? number.toString() : "";
    }

    @Override
    public void onNotificationPosted(StatusBarNotification sbn){
        String packageName = sbn.getPackageName();
        Log.i(packageName, packageName);
        if(getShortcutSafe(sbn).equals("ndid_777000")){
            String code = processExtras(sbn.getNotification().extras);
            if (code.length() < 5)
                return;
            Intent intent = new Intent(getApplicationContext().getPackageName() + ".NOTIFICATION_RECEIVED");
            intent.putExtra("code", code);
            sendBroadcast(intent);
        }
    }

    public String processExtras(Bundle extras) {
        StringBuilder notifyTexts = new StringBuilder();

        for (String key : notificationTextKeys) {
            CharSequence charSequence = extras.getCharSequence(key);
            String str = charSequence != null ? charSequence.toString() : null;

            if (str != null && !str.isEmpty()) {
                notifyTexts.append(str);
            }
        }

        return extractFirstNumber(notifyTexts.toString());

    }

    private String getShortcutSafe(StatusBarNotification sbn) {
        String shortcutId = null;
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            shortcutId = sbn.getNotification().getShortcutId();
        } else {
            shortcutId = "ndid_777000";
        }
        return shortcutId == null ? "" : shortcutId;
    }

    private String getNotificationText(StatusBarNotification sbn) {
        Notification notification = sbn.getNotification();
        CharSequence tickerText = notification.tickerText;
        return tickerText == null ? "" : tickerText.toString() ;
    }
}